package com.ruoyi.bst.store.service;

import java.util.List;
import com.ruoyi.bst.store.domain.Store;
import com.ruoyi.bst.store.domain.StoreVO;
import com.ruoyi.bst.store.domain.StoreQuery;
import org.apache.ibatis.annotations.Param;

/**
 * 店铺Service接口
 *
 * @author ruoyi
 * @date 2025-04-18
 */
public interface StoreService
{
    /**
     * 查询店铺
     *
     * @param id 店铺主键
     * @return 店铺
     */
    public StoreVO selectStoreById(Long id, boolean scope);

    default StoreVO selectStoreById(Long id) {
        return this.selectStoreById(id, false);
    }

    public StoreVO selectOne(StoreQuery query);

    /**
     * 查询店铺列表
     *
     * @param query 店铺
     * @return 店铺集合
     */
    public List<StoreVO> selectStoreList(StoreQuery query);

    /**
     * 新增店铺
     *
     * @param store 店铺
     * @return 结果
     */
    public int insertStore(Store store);

    /**
     * 修改店铺
     *
     * @param store 店铺
     * @return 结果
     */
    public int updateStore(Store store);

    /**
     * 批量删除店铺
     *
     * @param storeIds 需要删除的店铺主键集合
     * @return 结果
     */
    public int deleteStoreByStoreIds(Long[] storeIds);

    /**
     * 删除店铺信息
     *
     * @param storeId 店铺主键
     * @return 结果
     */
    public int deleteStoreByStoreId(Long storeId);

    public int logicDel(List<Long> ids);

}
